/*
var a =  [
           [ 1.0, 1.0, 1.0 ],
           [ 1.0, 1.0, 1.0 ],
           [ 1.0, 1.0, 1.0 ]
         ];

var b =  [
           [ 1.0, 2.0, 3.0 ],
           [ 4.0, 5.0, 6.0 ],
           [ 7.0, 8.0, 9.0 ]
         ];
*/

var n = 1000;

var a = makeRandomMatrix(n);
var b = makeRandomMatrix(n)

console.time("multiply");
var c = multiply(a, b);
console.timeEnd("multiply");


function multiply(a, b)
  {
    if (!checkArgs(a, b))
        throw ("Matrices not compatible for multiplication");

    var nRows = a.length;
    var nCols = b[0].length;

    var result = Array(nRows);

    for(var rowNum = 0;  rowNum < nRows; ++rowNum)
      {
        result[rowNum] = Array(nCols);

        for(var colNum = 0;  colNum < nCols;  ++colNum)
          {
            var sum = 0;

            for(var i = 0;  i < a[0].length;  ++i)
                sum += a[rowNum][i]*b[i][colNum];

            result[rowNum][colNum] = sum;
          }
      }

    return result;
  }


/**
 * Checks that the two specified matrices are compatible for multiplication.
 * @return true if the matrix product a*b is possible;
 */
function checkArgs(a, b)
  {
    // check that matrices are not null
    if (a == null || b ==  null)
        return false;

    // check that we actually have columns
    if (a[0] == null || b[0] == null)
        return false;

    // check that all columns of matrix a have the same size
    var aColSize = a[0].length;
    for (var i = 1;  i < a.length;  ++i)
      {
        if (a[i] == null || a[i].length != aColSize)
            return false;
      }

    // check that all columns of matrix b have the same size
    var bColSize = b[0].length;
    for (var i = 1;  i < b.length;  ++i)
      {
        if (b[i] == null || b[i].length != bColSize)
            return false;
      }

    // check that number of columns of matrix a equals the number of rows of matrix b
    if (aColSize != b.length)
        return false;

    return true;
  }


/**
 * Creates a matrix of random numbers
 */
function makeRandomMatrix(n)
  {
    var result = Array(n);

    for (var i = 0;  i < n;  ++i)
      {
        result[i] = Array(n);

        for (var j = 0;  j < n;  ++j)
            result[i][j] = Math.random();
      }

    return result;
  }


/**
 * Prints a 2-dimensional array of numbers in standard matrix form.
 * (Used for testing.)
 */
function printMatrix(a)
  {
    var util = require("util");

    for (var i = 0;  i < a.length;  ++i)
      {
        for (var j = 0;  j < a[i].length;  ++j)
            util.print(a[i][j], " ");

        util.print("\n");
      }

    util.print("\n");
  }
